@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Interface for flopy's implementation for MODFLOW
+"""Interface for flopy's implementation for MODFLOW."""
 
 import geopandas
 import numpy as np
@@ -17,12 +17,12 @@ except ImportError:
 
 from swn.base import SurfaceWaterNetwork
 from swn.logger import get_logger
-from swn.spatial import get_sindex
+from swn.spatial import get_crs, get_sindex, compare_crs
 from swn.util import abbr_str
 
 
 class MfSfrNetwork(object):
-    """MODFLOW SFR network
+    """MODFLOW SFR network class.
 
     Attributes
     ----------
@@ -46,6 +46,7 @@ class MfSfrNetwork(object):
         abstraction/diversion identifiers, where iupseg != 0.
     logger : logging.Logger
         Logger to show messages.
+
     """
 
     def __init__(self, swn, model, ibound_action='freeze',
@@ -56,7 +57,7 @@ class MfSfrNetwork(object):
                  abstraction={}, inflow={},
                  flow={}, runoff={}, etsw={}, pptsw={},
                  logger=None):
-        """Create a MODFLOW SFR structure from a surface water network
+        """Create a MODFLOW SFR structure from a surface water network.
 
         Parameters
         ----------
@@ -119,6 +120,7 @@ class MfSfrNetwork(object):
             See generate_segment_data. Default is {} (zero).
         logger : logging.Logger, optional
             Logger to show messages.
+
         """
         if logger is None:
             logger = get_logger(self.__class__.__name__)
@@ -146,11 +148,15 @@ class MfSfrNetwork(object):
         segments_crs = getattr(self.segments.geometry, 'crs', None)
         modelgrid_crs = None
         modelgrid = model.modelgrid
-        if modelgrid.proj4 is not None:
-            from fiona import crs as fiona_crs
-            modelgrid_crs = fiona_crs.from_string(modelgrid.proj4)
+        epsg = modelgrid.epsg
+        proj4_str = modelgrid.proj4
+        if epsg is not None:
+            segments_crs, modelgrid_crs, same = compare_crs(segments_crs, epsg)
+        else:
+            segments_crs, modelgrid_crs, same = compare_crs(segments_crs,
+                                                            proj4_str)
         if (segments_crs is not None and modelgrid_crs is not None and
-                segments_crs != modelgrid_crs):
+                not same):
             self.logger.warning(
                 'CRS for segments and modelgrid are different: {0} vs. {1}'
                 .format(segments_crs, modelgrid_crs))
@@ -630,6 +636,9 @@ class MfSfrNetwork(object):
         # Add diversions (i.e. SW takes)
         if swn.diversions is not None:
             self.diversions = swn.diversions.copy()
+            # Mark diversions that are not used / outside model
+            self.diversions['in_model'] = True
+            outside_model = []
             # Add columns for ICALC=0
             self.segment_data['depth1'] = 0.0
             self.segment_data['depth2'] = 0.0
@@ -639,7 +648,6 @@ class MfSfrNetwork(object):
                 isinstance(self.diversions, geopandas.GeoDataFrame) and
                 'geometry' in self.diversions.columns and
                 (~self.diversions.is_empty).all())
-            drop_divids = []
             if swn.has_z:
                 empty_geom = wkt.loads('linestring z empty')
             else:
@@ -647,7 +655,7 @@ class MfSfrNetwork(object):
             for divid, divn in self.diversions.iterrows():
                 if divn.from_segnum not in segnum2nseg_d:
                     # segnum does not exist -- segment is outside model
-                    drop_divids.append(divid)
+                    outside_model.append(divid)
                     continue
                 iupseg = segnum2nseg_d[divn.from_segnum]
                 assert iupseg != 0, iupseg
@@ -691,15 +699,17 @@ class MfSfrNetwork(object):
                         bbox_match = sorted(
                             grid_sindex.nearest(divn.geometry.bounds))
                         # more than one nearest can exist! just take one...
+                        num_found = len(bbox_match)
                         grid_cell = grid_cells.iloc[bbox_match[0]]
-                        if len(bbox_match) > 1:
-                            self.logger.warning(
-                                '%d grid cells are nearest to diversion %r, '
-                                'but only taking the first %s',
-                                len(bbox_match), divid, grid_cell)
                     else:  # slow scan of all cells
-                        raise NotImplementedError(
-                            'expected a grid spatial index')
+                        sel = grid_cells.intersects(divn.geometry)
+                        num_found = sel.sum()
+                        grid_cell = grid_cells.loc[sel].iloc[0]
+                    if num_found > 1:
+                        self.logger.warning(
+                            '%d grid cells are nearest to diversion %r, '
+                            'but only taking the first %s',
+                            num_found, divid, grid_cell)
                     row, col = grid_cell.name
                     strtop = dis.top[row, col]
                     reach_d.update({
@@ -720,12 +730,12 @@ class MfSfrNetwork(object):
                 seg_d.update({'depth1': depth, 'depth2': depth})
                 self.reaches.loc[len(self.reaches) + 1] = reach_d
                 self.segment_data.loc[nseg] = seg_d
-            if drop_divids:
-                self.diversions.drop(drop_divids, axis=0, inplace=True)
+            if outside_model:
+                self.diversions.loc[list(outside_model), 'in_model'] = False
                 self.logger.debug(
-                    'added %d diversions, dropped %d that did not connect to '
+                    'added %d diversions, ignoring %d that did not connect to '
                     'existing segments',
-                    len(self.diversions), len(drop_divids))
+                    self.diversions['in_model'].sum(), len(outside_model))
             else:
                 self.logger.debug(
                     'added all %d diversions', len(self.diversions))
@@ -746,6 +756,7 @@ class MfSfrNetwork(object):
             model=model, reach_data=reach_data, segment_data=segment_data)
 
     def __repr__(self):
+        """Return string representation of MfSfrNetwork object."""
         is_diversion = self.segment_data['iupseg'] != 0
         segnum_l = list(self.segment_data.loc[~is_diversion, 'segnum'])
         segments_line = str(len(segnum_l)) + ' from segments'
@@ -758,7 +769,7 @@ class MfSfrNetwork(object):
             diversions_line = str(len(divid_l)) + ' from diversions'
             if set(divid_l) != set(self.diversions.index):
                 diversions_line += ' ({:.0%} used)'.format(
-                    len(divid_l), len(divid_l) / float(len(self.diversions)))
+                    len(divid_l) / float(len(self.diversions)))
             diversions_line += abbr_str(divid_l, 4)
         else:
             diversions_line = 'no diversions'
@@ -783,7 +794,7 @@ class MfSfrNetwork(object):
     def plot(self, column='iseg',
              cmap='viridis_r', legend=False):
         """
-        Shows map of reaches with inflow segments in royalblue
+        Show map of reaches with inflow segments in royalblue.
 
         Parameters
         ----------
@@ -798,6 +809,7 @@ class MfSfrNetwork(object):
         Returns
         -------
         AxesSubplot
+
         """
         import matplotlib.pyplot as plt
 
@@ -828,7 +840,7 @@ class MfSfrNetwork(object):
         return ax
 
     def get_reach_data(self):
-        """Returns numpy.recarray for flopy's ModflowSfr2 reach_data
+        """Return numpy.recarray for flopy's ModflowSfr2 reach_data.
 
         Parameters
         ----------
@@ -837,6 +849,7 @@ class MfSfrNetwork(object):
         Returns
         -------
         numpy.recarray
+
         """
         from flopy.modflow.mfsfr2 import ModflowSfr2
         # Build reach_data for Data Set 2
@@ -849,7 +862,8 @@ class MfSfrNetwork(object):
 
     def set_segment_data(self, abstraction={}, inflow={}, flow={}, runoff={},
                          etsw={}, pptsw={}, return_dict=False):
-        """Set timeseries data in segment_data required for flopy's ModflowSfr2
+        """
+        Set timeseries data in segment_data required for flopy's ModflowSfr2.
 
         This method does two things:
 
@@ -893,6 +907,7 @@ class MfSfrNetwork(object):
         Returns
         -------
         None or dict (if return_dict is True)
+
         """
         from flopy.modflow.mfsfr2 import ModflowSfr2
         # Build stress period DataFrame from modflow model
@@ -913,8 +928,10 @@ class MfSfrNetwork(object):
             diversions_divids = set()
 
         def check_ts(data, name):
-            """Returns DataFrame with index along nper and columns for
-            either segnum or divid (checked later)"""
+            """Return DataFrame with index along nper.
+
+            Columns are either segnum or divid (checked later).
+            """
             if isinstance(data, dict):
                 data = pd.DataFrame(data, index=stress_df['start'])
             elif not isinstance(data, pd.DataFrame):
@@ -1145,9 +1162,7 @@ class MfSfrNetwork(object):
             self.model.sfr.segment_data = segment_data
 
     def get_seg_ijk(self):
-        """
-        This will just get the upstream and downstream segment k,i,j
-        """
+        """Get the upstream and downstream segment k,i,j."""
         topidx = self.reaches['ireach'] == 1
         kij_df = self.reaches[topidx][['iseg', 'k', 'i', 'j']].sort_values(
             'iseg')
@@ -1174,6 +1189,7 @@ class MfSfrNetwork(object):
     def get_top_elevs_at_segs(self, m=None):
         """
         Get topsurface elevations associated with segment up and dn elevations.
+
         Adds elevation of model top at
         upstream and downstream ends of each segment
         :param m: modeflow model with active dis package
@@ -1190,7 +1206,8 @@ class MfSfrNetwork(object):
 
     def get_segment_incision(self):
         """
-        Calculates the upstream and downstream incision of the segment
+        Calculate the upstream and downstream incision of the segment.
+
         :return:
         """
         self.segment_data['diff_up'] = (self.segment_data['top_up'] -
@@ -1201,8 +1218,8 @@ class MfSfrNetwork(object):
 
     def set_seg_minincise(self, minincise=0.2, max_str_z=None):
         """
-        Set segment elevation so that they have the minumum incision
-        from the top surface
+        Set segment elevation to have the minumum incision from the top.
+
         :param minincise: Desired minimum incision
         :param max_str_z: Optional parameter to prevent streams at
         high elevations (forces incision to max_str_z)
@@ -1225,7 +1242,8 @@ class MfSfrNetwork(object):
 
     def get_segment_length(self):
         """
-        Get segment length from accumulated reach lengths
+        Get segment length from accumulated reach lengths.
+
         :return:
         """
         # extract segment length for calculating minimun drop later
@@ -1235,16 +1253,16 @@ class MfSfrNetwork(object):
         return seglen
 
     def get_outseg_elev(self):
-        """Get the maximum elevup associated with all downstream segments
-        for each segment"""
+        """Get the max elevup from all downstream segments for each segment."""
         self.segment_data['outseg_elevup'] = self.segment_data.outseg.apply(
             lambda x: self.segment_data.loc[
                 self.segment_data.index == x].elevup).max(axis=1)
         return self.segment_data['outseg_elevup']
 
     def set_outseg_elev_for_seg(self, seg):
-        """
-        gets all the defined outseg_elevup associated with a specific segment
+        """Set outseg elevation for segment.
+
+        Gets all the defined outseg_elevup associated with a specific segment
         (multiple upstream segements route to one segment)
         Returns a df with all the calculated outseg elevups for each segment.
         .min(axis=1) is a good way to collapse to a series
@@ -1262,6 +1280,7 @@ class MfSfrNetwork(object):
     def minslope_seg(self, seg, *args):
         """
         Force segment to have minumim slope (check for backward flowing segs).
+
         Moves downstream end down (vertically, more incision)
         to acheive minimum slope.
         :param seg: Pandas Series containing one row of seg_data dataframe
@@ -1309,7 +1328,8 @@ class MfSfrNetwork(object):
                           'outseg_elevup': outseg_up})
 
     def set_forward_segs(self, min_slope=1.e-4):
-        """
+        """Set minimum slope in forwards direction.
+
         Ensure slope of all segment is at least min_slope
         in the downstream direction.
         Moves down the network correcting downstream elevations if necessary
@@ -1350,7 +1370,8 @@ class MfSfrNetwork(object):
     def fix_segment_elevs(self, min_incise=0.2, min_slope=1.e-4,
                           max_str_z=None):
         """
-        Wrapper function for calculating SFR segment elevations.
+        Provide wrapper function for calculating SFR segment elevations.
+
         Calls series of functions to process and move sfr segment elevations,
         to try to ensure:
             0. Segments are below the model top
@@ -1388,11 +1409,13 @@ class MfSfrNetwork(object):
 
     def reconcile_reach_strtop(self):
         """
-        recalculate reach strtop elevations after moving segment elevations
+        Recalculate reach strtop elevations after moving segment elevations.
+
         :return: None
         """
         def reach_elevs(seg):
-            """
+            """Return reach properties.
+
             Calculate reach elevation from segment slope and
             reach length along segment.
             :param seg: one row of reach data dataframe grouped by segment
@@ -1422,7 +1445,8 @@ class MfSfrNetwork(object):
 
     def set_topbot_elevs_at_reaches(self, m=None):
         """
-        get top and bottom elevation of the cell containing a reach
+        Get top and bottom elevation of the cell containing a reach.
+
         :param m: Modflow model
         :return: dataframe with reach cell top and bottom elevations
         """
@@ -1435,8 +1459,9 @@ class MfSfrNetwork(object):
         return self.reaches[['top', 'bot']]
 
     def fix_reach_elevs(self, minslope=0.0001, fix_dis=True, minthick=0.5):
-        """
-        need to ensure reach elevation is:
+        """Fix reach elevations.
+
+        Need to ensure reach elevation is:
             0. below the top
             1. below the upstream reach
             2. above the minimum slope to the bottom reach elevation
@@ -1445,6 +1470,24 @@ class MfSfrNetwork(object):
 
         :return:
         """
+        def _check_reach_v_laybot(r, botms, buffer=1.0, rbed_elev=None):
+            if rbed_elev is None:
+                rbed_elev = r.strtop - r.strthick
+            if (rbed_elev - buffer) < r.bot:
+                # if new strtop is below layer one
+                # drop bottom of layer one to accomodate stream
+                # (top, bed thickness and buffer)
+                new_elev = rbed_elev - buffer
+                print('seg {} reach {} @ {} '
+                      'is below layer 1 bottom @ {}'
+                      .format(seg, r.ireach, rbed_elev,
+                              r.bot))
+                print('    dropping layer 1 bottom to {} '
+                      'to accommodate stream @ i = {}, j = {}'
+                      .format(new_elev, r.i, r.j))
+                botms[0, r.i, r.j] = new_elev
+            return botms
+
         buffer = 1.0  # 1 m (buffer to leave at the base of layer 1 -
         # also helps with precision issues)
         # make sure elevations are up-to-date
@@ -1453,6 +1496,7 @@ class MfSfrNetwork(object):
         _ = self.set_topbot_elevs_at_reaches()
         # top read from dis as float32 so comparison need to be with like
         reachsel = self.reaches['top'] <= self.reaches['strtop']
+        reach_ij = tuple(self.reaches[['i', 'j']].values.T)
         print('{} segments with reaches above model top'.format(
             self.reaches[reachsel]['iseg'].unique().shape[0]))
         # get segments with reaches above the top surface
@@ -1472,20 +1516,11 @@ class MfSfrNetwork(object):
             rsel = self.reaches['iseg'] == seg
             segsel = self.segment_data.index == seg
 
-            if seg in segsabove:
+            if seg in segsabove.index:
                 # check top and bottom reaches are above layer 1 bottom
                 # (not adjusting elevations of reaches)
                 for reach in self.reaches[rsel].iloc[[0, -1]].itertuples():
-                    if reach.strtop - reach.strthick < reach.bot + buffer:
-                        # drop bottom of layer one to accomodate stream
-                        new_elev = reach.strtop - reach.strthick - buffer
-                        print('seg {} reach {} is below layer 1 bottom'
-                              .format(seg, reach.ireach))
-                        print(
-                            'dropping layer 1 bottom to {} to accomodate '
-                            'stream @ i = {}, j = {}'
-                            .format(new_elev, reach.i, reach.j))
-                        layerbots[0, reach.i, reach.j] = new_elev
+                    layerbots = _check_reach_v_laybot(reach, layerbots, buffer)
                 # apparent optimised incision based
                 # on the incision gradient for the segment
                 self.reaches.loc[rsel, 'strtop_incopt'] = \
@@ -1591,16 +1626,8 @@ class MfSfrNetwork(object):
                     # check if new stream top is above layer 1 with a buffer
                     # (allowing for bed thickness)
                     reachbed_elev = upreach_strtop - reach.strthick
-                    if (reachbed_elev - buffer) < reach.bot:
-                        # if new strtop is below layer one
-                        # drop bottom of layer one to accomodate stream
-                        # (top, bed thickness and buffer)
-                        new_elev = reachbed_elev - buffer
-                        print('    dropping layer 1 bottom from {} to {} '
-                              'to accommodate stream @ i = {}, j = {}'
-                              .format(reachbed_elev, new_elev,
-                                      reach.i, reach.j))
-                        layerbots[0, reach.i, reach.j] = new_elev
+                    layerbots = _check_reach_v_laybot(reach, layerbots, buffer,
+                                                      reachbed_elev)
                     upreach_cmid = reach.cmids
                     # upreach_slope=reach.slope
             else:
@@ -1610,19 +1637,12 @@ class MfSfrNetwork(object):
                       .format(seg))
                 for reach in self.reaches[rsel].itertuples():
                     reachbed_elev = reach.strtop - reach.strthick
-                    if (reachbed_elev - buffer) < reach.bot:
-                        # strtop is below layer one
-                        # drop bottom of layer one to accommodate stream
-                        # (top, bed thickness and buffer)
-                        new_elev = reachbed_elev - buffer
-                        print('seg {} reach {} @ {} '
-                              'is below layer 1 bottom @ {}'
-                              .format(seg, reach.ireach, reachbed_elev,
-                                      reach.bot))
-                        print('    dropping layer 1 bottom to {} '
-                              'to accommodate stream @ i = {}, j = {}'
-                              .format(new_elev, reach.i, reach.j))
-                        layerbots[0, reach.i, reach.j] = new_elev
+                    layerbots = _check_reach_v_laybot(reach, layerbots, buffer,
+                                                      reachbed_elev)
+            # OH CRAP need to update dis bottoms in reach df!
+            # self.reaches['top'] = layerbots[
+            #     tuple(self.reaches[['i', 'j']].values.T)]
+            self.reaches['bot'] = layerbots[0][reach_ij]
         if fix_dis:
             # fix dis for incised reaches
             for lay in range(self.model.dis.nlay - 1):
@@ -1638,14 +1658,16 @@ class MfSfrNetwork(object):
 
     def sfr_plot(self, model, sfrar, dem, points=None, points2=None,
                  label=None):
+        """Plot sfr."""
         p = ModelPlot(model)
         p._add_plotlayer(dem, label="Elevation (m)")
-        p._add_sfr(sfrar, cat_cmap=False, colorbar=True,
-                   cbar_label=label)
+        p._add_sfr(sfrar, cat_cmap=False, cbar=True,
+                   label=label)
         return p
 
     def plot_reaches_above(self, model, seg, dem=None,
                            plot_bottom=False, points2=None):
+        """Plot sfr reaches above."""
         # ensure reach elevations are up-to-date
         _ = self.set_topbot_elevs_at_reaches()
         dis = model.dis
@@ -1692,14 +1714,13 @@ class MfSfrNetwork(object):
 
 
 class ModelPlot(object):
-    """
-    Object for plotting array style results
-    """
+    """Object for plotting array style results."""
 
     def __init__(self, model, domain_extent=None, fig=None, ax=None,
                  figsize=None):
         """
-        Container to help with results plotting functions
+        Container to help with results plotting functions.
+
         :param model: flopy modflow model object
         :param fig: matplotlib figure handle
         :param ax: Cartopy GeoAxes with .projection attribute
@@ -1710,22 +1731,20 @@ class ModelPlot(object):
         from matplotlib import pyplot as plt
 
         self.model = model
-        # Use model projection, if defined by either proj4 or epsg attrs
-        proj_str = model.modelgrid.proj4
+        self.fig = fig
+        self.ax = ax
+        # Use model projection, if defined by either epsg or proj4 attrs
         epsg = model.modelgrid.epsg
-        self.pprj = None
-        try:
-            if proj_str:
-                self.pprj = pyproj.Proj(proj_str)
-            elif epsg:
-                self.pprj = pyproj.Proj('epsg:' + str(epsg))
-        except RuntimeError:
-            pass
+        self.pprj = get_crs(epsg)
+        if self.pprj is None:
+            self.pprj = get_crs(model.modelgrid.proj4)
         if domain_extent is None and self.pprj is not None:
             xmin, xmax, ymin, ymax = self.model.modelgrid.extent
-            assert not self.pprj.is_latlong()
-            lonmin, latmin = self.pprj(xmin, ymin, inverse=True)
-            lonmax, latmax = self.pprj(xmax, ymax, inverse=True)
+            assert self.pprj.is_projected, self.pprj
+            latlon = pyproj.CRS.from_epsg(4326)  # axis order: lat, lon
+            tfm = pyproj.Transformer.from_crs(self.pprj, latlon)
+            latmin, lonmin = tfm.transform(xmin, ymin)
+            latmax, lonmax = tfm.transform(xmax, ymax)
             self.domain_extent = [lonmin, lonmax, latmin, latmax]
         else:
             self.domain_extent = domain_extent
@@ -1744,18 +1763,20 @@ class ModelPlot(object):
                 try:
                     import cartopy.crs as ccrs
                     self.mprj = ccrs.epsg(epsg)
+                    # empty figure container cartopy geoaxes
+                    self.fig, self.ax = plt.subplots(figsize=figsize,
+                                                     subplot_kw=dict(
+                                                         projection=self.mprj))
                 except ImportError:
-                    pass
-            # empty figure container cartopy geoaxes
-            self.fig, self.ax = plt.subplots(figsize=figsize,
-                                             subplot_kw=dict(
-                                                 projection=self.mprj))
+                    self.fig, self.ax = plt.subplots(figsize=figsize)
         else:
-            self.fig = fig
-            self.ax = ax
-            self.mprj = self.ax.projection  # map projection
+            try:
+                self.mprj = self.ax.projection  # map projection
+            except NameError:
+                self.mprj = None
         # self._get_base_ax()
-        self._set_divider()
+        if self.ax is not None:
+            self._set_divider()
 
     # def _get_base_ax(self):
     #     """
@@ -1813,7 +1834,8 @@ class ModelPlot(object):
         return vmin, vmax
 
     def _get_cbar_props(self):
-        """
+        """Get colorbar properties.
+
         Get properties for next colorbar and its label based on the number of
         axis already in plot
 
@@ -1837,15 +1859,14 @@ class ModelPlot(object):
         return divider_props, props
 
     def _set_divider(self):
-        """
-        initiate a mpl divider for the colorbar location
-        """
+        """Initiate a mpl divider for the colorbar location."""
         from mpl_toolkits.axes_grid1 import make_axes_locatable
         self.divider = make_axes_locatable(self.ax)
 
     def _add_ibound_mask(self, lay, zorder=20, alpha=0.5):
         """
-        Add the ibound to plot
+        Add the ibound to plot.
+
         :param lay: layer for selecting model ibound
         :param zorder: mpl plotting overlay order
         :param alpha: mpl transparency
@@ -1857,37 +1878,58 @@ class ModelPlot(object):
             array, extent=self.extent, transform=self.mprj, cmap="Greys_r",
             origin="upper", zorder=zorder, alpha=alpha)
 
-    def _add_plotlayer(self, k, zorder=10, alpha=0.8, label=None,
-                       get_range=False):
+    def _add_plotlayer(self, ar, vmin=None, vmax=None, norm=None, cmap=None,
+                       zorder=10, alpha=0.8, cbar=True, label=None):
         """
-        Add image for head
-        :param k: 2D numpy array
+        Add image for layer array
+
+        :param ar: 2D numpy array to plot
+        :param vmin: minimum value to clip
+        :param vmax: maximum value to clip
+        :param norm: mpl normalizer (default is None)
+        :param cmap: mpl colormap (default is 'plasma')
         :param zorder: mpl overlay order
         :param alpha: mpl transparency
+        :param cbar: flag to plot with colorbar for layer
+        :param label: label for colorbar
         """
-        from matplotlib import pyplot as plt
 
-        if get_range:
-            vmin, vmax = self._get_range(k)
+        if self.ax is None:
+            return
+
+        from matplotlib import pyplot as plt
+        from matplotlib import cm
+        if cmap is None:
+            cmap = cm.get_cmap('viridis')
+        if self.mprj is None:
+            transform = self.ax.transData
         else:
-            vmin, vmax = [None, None]
+            transform = self.mprj
         if label is None:
             print("No label passed for colour bar")
             label = ""
-        hax = self.ax.imshow(
-            k, zorder=zorder, extent=self.extent, origin="upper",
-            transform=self.mprj, alpha=alpha, vmin=vmin, vmax=vmax)
-        divider_props, props = self._get_cbar_props()
-        cax = self.divider.append_axes(
-            "right", size="5%", axes_class=plt.Axes, **divider_props)
-        cbar1 = self.fig.colorbar(hax, cax=cax)
-        cbar1.set_label(label, **props)
+        hax = self.ax.imshow(ar, zorder=zorder, vmin=vmin, vmax=vmax,
+                             extent=self.extent, origin="upper",
+                             transform=transform, norm=norm,
+                             alpha=alpha, cmap=cmap)
+        if cbar:
+            if label is None:
+                print("No label passed for colour bar")
+                label = ""
+            divider_props, props = self._get_cbar_props()
+            cax = self.divider.append_axes("right", size="5%",
+                                           axes_class=plt.Axes,
+                                           **divider_props)
+            cbar1 = self.fig.colorbar(hax, cax=cax)
+            cbar1.set_label(label, **props)
+        return hax
 
-    def _add_sfr(self, x, zorder=11, colorbar=True, cat_cmap=False,
-                 cbar_label=None, cmap_txt='bwr_r',
+    def _add_sfr(self, x, zorder=11, cbar=True, cat_cmap=False,
+                 label=None, cmap_txt='bwr_r',
                  points=None, points2=None):
         """
-        Plot the array of surface water exchange (with SFR)
+        Plot the array of surface water exchange (with SFR).
+
         :param x: 2D numpy array
         :param zorder: mpl overlay order
         """
@@ -1907,9 +1949,8 @@ class ModelPlot(object):
         else:
             cmap = cm.get_cmap(cmap_txt)
             norm = MidpointNormalize(vmin=vmin, vmax=vmax, midpoint=0)
-        imx = self.ax.imshow(x, extent=self.extent, transform=self.mprj,
-                             cmap=cmap, norm=norm, vmin=vmin, vmax=vmax,
-                             zorder=zorder, origin='upper')
+        imx = self._add_plotlayer(x, cmap=cmap, norm=norm, zorder=zorder,
+                                  alpha=1, label=label, cbar=cbar)
         if points is not None:
             self.ax.scatter(self.model.modelgrid.xcellcenters[
                                 points.i, points.j],
@@ -1924,26 +1965,24 @@ class ModelPlot(object):
                                 points2.i, points2.j],
                             marker='o', zorder=15, facecolors='none',
                             edgecolors='b')
-        if colorbar:
-            divider_props, props = self._get_cbar_props()
-            cax = self.divider.append_axes("right", size="5%",
-                                           axes_class=plt.Axes,
-                                           **divider_props)
-            cbar1 = self.fig.colorbar(imx, cax=cax)
-            cbar1.set_label(cbar_label, **props)
+        if cbar:
             if cat_cmap:
-                # imsfr.set_clim(1, n + 1)
-                cbar1.set_ticks(bounds[:-1] + np.diff(bounds) / 2)
-                cbar1.set_ticklabels(np.sort(vals).astype(int))
+                last_cbar = self.fig.get_axes()[-1]
+                last_cbar.set_yticks(bounds[:-1] + np.diff(bounds) / 2)
+                last_cbar.set_yticklabels(np.sort(vals).astype(int))
 
 
 if matplotlib:
     class MidpointNormalize(matplotlib.colors.Normalize):
+        """Mid-point normalize class."""
+
         def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
+            """Initialize method."""
             self.midpoint = midpoint
             matplotlib.colors.Normalize.__init__(self, vmin, vmax, clip)
 
         def __call__(self, value, clip=None):
+            """Call method."""
             # I'm ignoring masked values and all kinds of edge cases to make a
             # simple example...
             x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
@@ -1952,9 +1991,7 @@ if matplotlib:
 
 
 def sfr_rec_to_df(sfr):
-    """
-    Convert flopy rec arrays for ds2 and ds6 to pandas dataframes
-    """
+    """Convert flopy rec arrays for ds2 and ds6 to pandas dataframes."""
     d = sfr.segment_data
     # multi index
     reform = {(i, j): d[i][j] for i in d.keys() for j in d[i].dtype.names}
@@ -1966,7 +2003,8 @@ def sfr_rec_to_df(sfr):
 
 def sfr_dfs_to_rec(model, segdatadf, reachdatadf, set_outreaches=False,
                    get_slopes=True, minslope=None):
-    """
+    """Convert sfr ds6 and ds2 to model sfr rec.
+
     Function to convert sfr ds6 (seg data) and ds2 (reach data) to model.sfr
     rec arrays option to update slopes from reachdata dataframes
     """
@@ -1996,7 +2034,7 @@ def sfr_dfs_to_rec(model, segdatadf, reachdatadf, set_outreaches=False,
 
 
 def geotransform_from_flopy(m):
-    """Returns GDAL-style geotransform from flopy model"""
+    """Return GDAL-style geotransform from flopy model."""
     try:
         import flopy
     except ImportError:
